;;======================================================================;;
;;				SIMPLE SERVO				;;
;;======================================================================;;
;;									;;
;; Program:         SERVO ANLG -- Driver for model servo		;;
;; Code:            Jindra Fuk					;;
;; Platform:        Microchip PIC10F200, 4 Mhz				;;
;; Date:            29.05.2015						;;
;; First release:   5.07.2015						;;
;; LastDate:        5.07.2015						;;
;; Updated:         Jindra Fuk					;;
;; UpdateDate:      5.07.2015						;;
;;									;;
;;======================================================================;;

; Minimal external components, uses internal oscilator at 4 MHz

; This program is distributed as is but WITHOUT ANY WARRANTY
; I hope you enjoy!!
;
; Revisions:
; 29.05.2015	Start of writing code
; 05.07.2015	Try to convert to 10F200
; 11.11.2016	extended to 10F206

; ----- Definitions

#define		__VERNUM	D'1'
#define		__VERDAY	0x11
#define		__VERMONTH	0x11
#define		__VERYEAR	0x16


	IFDEF __10F200
	LIST	   p=10F200	; target processor
	#include p10f200.inc
	ENDIF

	IFDEF __10F206
	LIST	   p=10F206	; target processor
	#include p10f206.inc
	ENDIF

	;errorlevel -227	; Substituting RETLW 0 for RETURN pseudo-op
	;errorlevel -302	; Register in operand not in bank 0. Ensure that bank bits are correct.
	;errorlevel -305	; Using default destination of 1 (file).



        __CONFIG   _MCLRE_OFF & _CP_OFF & _WDT_ON

				; Make sure that internal osc. is calibrated
				; Value has to be read before reprogramming the device.

;          SOT23-6                            DIP8
;      +------U------+                  +------U------+
;     1|GP0       GP3|6 i INPUT        1|NC        GP3|8 i INPUT
; GND 2|Vss       Vdd|5 +5V        +5V 2|Vdd       Vss|7 GND
;     3|GP1       GP2|4 SERVO    SERVO 3|GP2        NC|6
;      +-------------+                 4|GP1       GP0|5
;                                       +-------------+

; --- Constant values

GP_TRIS         equ     b'00001011'		; GP2: servo out, 
;GP_INI          equ     0x00			; all zero

OPTION_INI	equ   b'11011110'   ; 11011110
                                ; 1-------, IOC off
                                ; -1------, weak pullups OFF
                                ; --0-----, T0CS source Fosc/4
                                ; ---1----, T0SE edge hi>lo
                                ; ----1---, PSA prescale WDT
                                ; -----110, PS prescaler 1:64

#define		SERVO	GPIO,2			; LED output
#define		IN1		GPIO,3			; Signal input

; --- EEPROM Section

; ----- Variables

; --- Internal RAM Section

        cblock  0x10

TEMP
COUNT	; Counter for spacing delay

CV0		; CV1 - first servo position (*10 in ms) - default 100-> 1000ms
CV1		; CV2 - second servo position (*10 in ms) - default 200-> 2000ms
CV2		; CV3 - Speed

CV2CNT	; - Speed counter

DEST	; destination of move for servo
PULSE		; current pulse duration

		endc


; --- Macros

#define		DNOP		goto	$+1		; do nothing in 2 steps (long nop)

; --------------- Program Section --------------------------------------


		org	0x000

start
		movwf   OSCCAL			;

		clrwdt
		movlw	OPTION_INI
		option					;
		movlw   GP_TRIS			;
		tris    GPIO			; GP3 input, all others outputs
		clrf    GPIO			; set output latches to '0'
	IFDEF __10F206
		bcf		CMCON0,CMPON	; disable comparator
	ENDIF

LoadCV
		call LoadCV0S
		movwf CV0
		call LoadCV1S
		movwf CV1
		call LoadCV2S
		movwf CV2
		
		movf	CV0,w
		btfsc	IN1
		movf	CV1,w
		movwf	PULSE
		movwf	DEST
		movlw	1
		movwf	CV2CNT		; 

; ----------------------------------------------------------------------

MainLoop:


		movf	PULSE,w		; 200: 2ms, 100: 1ms
		movwf	TEMP

		bsf	SERVO
PulseLoop:
		DNOP			; 1,2
		DNOP			; 3,4
		DNOP			; 5,6
		clrwdt			; 7
		decfsz	TEMP,f	; 8
		goto	PulseLoop	; 9,10

		bcf	SERVO
		
		decfsz	CV2CNT,f		; next speed step
		goto	Spacing			; yes, then do nothing

		movf	CV2,w               ; finished = prepare next step
		movwf	CV2CNT
                 
		movf	PULSE,w
		subwf   DEST,w			; dest - pulse --> w ( if dest<pulse then C, if dest=pulse then Z)

		btfss	STATUS,C		; going down ???
		goto	EndServoDec		; yes - then decrease
                 
		btfss	STATUS,Z		; going up ???
		goto	EndServoInc		; yes - then increase

		movlw	0x01
		movwf	CV2CNT

		goto	Spacing			; end of pulse calculation, start spacing

EndServoDec:
		decf	PULSE,f
		btfss	CV2,7			; if bit 7 is set, then handle it
		goto	Spacing			; end of pulse calculation, start spacing
		movlw	1				; set speed factor to 1
		movwf	CV2CNT
		movf	CV2,w				
		andlw	0x7F					; remove marking one from CV515
		subwf	PULSE,f				; pulse1 = pulse1 - cv515
		goto	Spacing			; end of pulse calculation, start spacing

EndServoInc:
		incf	PULSE,f
		btfss	CV2,7				; if bit 7 is set, then handle it
		goto	Spacing			; end of pulse calculation, start spacing
		movlw	1						; set speed factor to 1
		movwf	CV2CNT
		movf	CV2,w				
		andlw	0x7F					; remove marking one from CV515
		addwf	PULSE,f				; pulse1 = pulse1 - cv515
		;goto	Spacing			; end of pulse calculation, start spacing
                 

Spacing:
		movf	CV0,w
		btfsc	IN1
		movf	CV1,w
		movwf	DEST
		movf	CV2,w
		btfss	STATUS,Z
		goto	SpacingDelay
		movf	DEST,w
		movwf	PULSE
		; optimal is to delay 20000 cycles minus cycles spent in pulse.
		; hmm it can be difficult to calculate, than will start with 20000-1500 (middle position) = 18500
		; hmm, and convert to hex as 0x4600 (17920 dec) ;) 17920d / 5d = 3584d = 0xE00 

SpacingDelay:
		movlw	0x0F
		movwf	TEMP

		clrf	COUNT
		;movlw	0x0E
		;movwf	COUNT

SpacingDelayLoop:
		clrwdt					;1
		nop		;2
		decfsz	COUNT,f		;3
		goto	SpacingDelayLoop	;4,5
		decfsz	TEMP,f	; 8
		goto	SpacingDelayLoop

		goto	MainLoop

LoadCV0S:				; call can be done only o short address
		goto LoadCV0	; goto cam be also to long
LoadCV1S:				; call can be done only o short address
		goto LoadCV1	; goto cam be also to long
LoadCV2S:				; call can be done only o short address
		goto LoadCV2	; goto cam be also to long


	IFDEF __10F322
		org	0x01F0
	ENDIF

	IFDEF __10F320
		org	0x00F0
	ENDIF

	IFDEF __10F200
		org	0x00F0
	ENDIF

	IFDEF __10F206
		org	0x01F0
	ENDIF

LoadCV0:
		retlw .100	; CV0 - servo position A (*10 in ms) - default 100-> 1000ms
LoadCV1:
		retlw .200	; CV1 - servo position B (*10 in ms) - default 200-> 2000ms
LoadCV2:
		retlw .001	; CV3 - Speed

	IFDEF __10F200
		org	0x00FF
		movlw 0xFF	; OSCCALL value here!!!!
	ENDIF

	IFDEF __10F206
		org	0x01FF
		movlw 0xFF	; OSCCALL value here!!!!
	ENDIF

	end